import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Properties;
import java.util.Map.Entry;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;

/**
 * 
 */

/**
 * @author meres
 * 
 */
public class PropertyEditor extends EditorPart {

	Properties propertyData;
	TableViewer viewer;
	boolean dirty;

	IStructuredContentProvider contentProvider = new IStructuredContentProvider() {
		Properties properties;

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			if (newInput instanceof Properties) {
				properties = (Properties) newInput;
			}
		}

		@Override
		public void dispose() {
		}

		@Override
		public Object[] getElements(Object inputElement) {
			return properties.entrySet().toArray();
		}
	};

	ColumnLabelProvider keyLabelProvider = new ColumnLabelProvider() {

		@SuppressWarnings("unchecked")
		@Override
		public String getText(Object element) {
			return ((Entry<String, String>) element).getKey();
		}

	};
	ColumnLabelProvider valueLabelProvider = new ColumnLabelProvider() {

		@SuppressWarnings("unchecked")
		@Override
		public String getText(Object element) {
			return ((Entry<String, String>) element).getValue();
		}

	};

	abstract class AbstractEditingSupport extends EditingSupport {

		TextCellEditor editor;

		public AbstractEditingSupport(ColumnViewer viewer) {
			super(viewer);
			editor = new TextCellEditor((Composite) viewer.getControl());
		}

		@Override
		protected boolean canEdit(Object element) {
			return true;
		}

		@Override
		protected CellEditor getCellEditor(Object element) {
			return editor;
		}

	}

	class KeyEditingSupport extends AbstractEditingSupport {
		public KeyEditingSupport(ColumnViewer viewer) {
			super(viewer);
		}

		@SuppressWarnings("unchecked")
		@Override
		protected Object getValue(Object element) {
			Entry<String, String> entry = (Entry<String, String>) element;
			return entry.getKey();
		}

		@SuppressWarnings("unchecked")
		@Override
		protected void setValue(Object element, Object value) {
			Entry<String, String> entry = (Entry<String, String>) element;
			String entryValue = (String)propertyData.get(entry.getKey());
			propertyData.remove(entry.getKey());
			propertyData.put(value, entryValue);
			viewer.setInput(propertyData);
			//viewer.update(propertyData.0, null)
			dirty = true;
			firePropertyChange(PROP_DIRTY);
		}
	}

	class ValueEditingSupport extends AbstractEditingSupport {
		public ValueEditingSupport(ColumnViewer viewer) {
			super(viewer);
		}

		@SuppressWarnings("unchecked")
		@Override
		protected Object getValue(Object element) {
			Entry<String, String> entry = (Entry<String, String>) element;
			return entry.getValue();
		}

		@SuppressWarnings("unchecked")
		@Override
		protected void setValue(Object element, Object value) {
			Entry<String, String> entry = (Entry<String, String>) element;
			entry.setValue(value.toString());
			viewer.update(element, null);
			dirty = true;
			firePropertyChange(PROP_DIRTY);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		try {
		FileEditorInput fileEditorInput = (FileEditorInput)getEditorInput();
		File file = fileEditorInput.getPath().toFile();
		monitor.beginTask("Saving file", propertyData.size());
		PrintStream ps;
		try {
			ps = new PrintStream(new FileOutputStream(file));
			for (Entry<Object, Object> property : propertyData.entrySet()) {
				ps.println(String.format("%s = %s", property.getKey(),property.getValue()));
				monitor.worked(1);
			}
		} finally {
			monitor.done();
		}
		//Notifying framework of file changes in the disk
		fileEditorInput.getFile().refreshLocal(PROP_TITLE, monitor);
		dirty = false;
		firePropertyChange(PROP_DIRTY);
		} catch (IOException e) {
			//TODO better logging here
			e.printStackTrace();
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite,
	 * org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		FileEditorInput file = (FileEditorInput) input;

		propertyData = new Properties();
		try {
			propertyData.load(new FileInputStream(file.getPath().toFile()));
		} catch (FileNotFoundException e) {
			throw new PartInitException(e.getMessage(), e);
		} catch (IOException e) {
			throw new PartInitException(e.getMessage(), e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return dirty;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.BORDER | SWT.FULL_SELECTION);

		viewer.setContentProvider(contentProvider);
		TableViewerColumn keyColumn = new TableViewerColumn(viewer, SWT.LEFT);
		keyColumn.getColumn().setText("Key");
		keyColumn.setLabelProvider(keyLabelProvider);
		keyColumn.setEditingSupport(new KeyEditingSupport(viewer));
		TableViewerColumn valueColumn = new TableViewerColumn(viewer, SWT.LEFT);
		valueColumn.getColumn().setText("Value");
		valueColumn.setLabelProvider(valueLabelProvider);
		valueColumn.setEditingSupport(new ValueEditingSupport(viewer));
		TableLayout tableLayout = new TableLayout();
		tableLayout.addColumnData(new ColumnWeightData(50, true));
		tableLayout.addColumnData(new ColumnWeightData(50, true));

		viewer.setInput(propertyData);

		// Formatting
		Table table = viewer.getTable();
		table.setLayout(tableLayout);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		getSite().setSelectionProvider(viewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void addElement() {
		int id=0;
		while (propertyData.containsKey("new" + id)) id++;
		propertyData.put("new" + id, "newvalue");
		dirty = true;
		firePropertyChange(PROP_DIRTY);
		viewer.setInput(propertyData);
	}
	
	public void removeSelectedElements(IStructuredSelection selection) {
		for (Object _element : selection.toArray()) {
			Entry<String, String> element = (Entry<String, String>) _element;
			if (propertyData.containsKey(element.getKey())) {
				propertyData.remove(element.getKey());
			}
		}
		dirty = true;
		firePropertyChange(PROP_DIRTY);
		viewer.refresh();
	}
}
